//Button interface functions

struct button {
   int minx;
   int miny;
   int maxx;
   int maxy;
   bool buttonstate;
   bool laststate;
   unsigned char activecolor;
   unsigned char inactivecolor;
   char * name;
   unsigned char textcolor;
   int textx;
   int texty;
};

void Initialize_Button(int minx, int miny, int maxx, int maxy,
                       bool buttonstate, unsigned char activecolor,
                       unsigned char inactivecolor, char * name,
                       unsigned char textcolor, button &Button) {
   Button.minx = minx;
   Button.miny = miny;
   Button.maxx = maxx;
   Button.maxy = maxy;
   Button.buttonstate = buttonstate;
   Button.laststate = buttonstate;
   Button.activecolor = activecolor;
   Button.inactivecolor = inactivecolor;
   Button.name = name;
   Button.textcolor = textcolor;
   Button.textx = minx+((maxx-minx)-strlen(name)*CHAR_WIDTH)/2;
   Button.texty = miny+((maxy-miny)-CHAR_HEIGHT)/2;


   if (Button.buttonstate) {
      Filled_Box(Button.minx, Button.miny, Button.maxx, Button.maxy,
                 Button.activecolor);
   } else {
      Filled_Box(Button.minx, Button.miny, Button.maxx, Button.maxy,
                 Button.inactivecolor);
   }
   Blit_String(Button.name, Button.textx, Button.texty,
               Button.textcolor, 0, true);
   return;
}

bool Process_Button(button &Button, int x, int y, bool b,
                    bool &depressed, bool forcedraw=false) {
   if ((Button.buttonstate != Button.laststate) || forcedraw) {
      Hide_Cursor();

      if (Button.buttonstate) {
         Filled_Box(Button.minx,Button.miny,Button.maxx,Button.maxy,
                    Button.activecolor);
      } else {
         Filled_Box(Button.minx,Button.miny,Button.maxx,Button.maxy,
                    Button.inactivecolor);
      }
      Blit_String(Button.name, Button.textx, Button.texty,
                     Button.textcolor, 0, true);
      Show_Cursor();
   }


   if (b && x >= Button.minx && x <= Button.maxx && y >= Button.miny
         && y <= Button.maxy && !depressed) {
      depressed = true;
      Hide_Cursor();
      if (Button.buttonstate) {
         Filled_Box(Button.minx,Button.miny,Button.maxx,Button.maxy,
                    Button.inactivecolor);
         Button.buttonstate=false;
      } else {
         Filled_Box(Button.minx,Button.miny,Button.maxx,Button.maxy,
                    Button.activecolor);
         Button.buttonstate=true;
      }
      Blit_String(Button.name, Button.textx, Button.texty,
                  Button.textcolor, 0, true);
      Show_Cursor();
   }

   Button.laststate = Button.buttonstate;
   return Button.buttonstate;
}

#define RADIOBUTTON  0x01 //A radio button set always has 1 button selected.
#define NORMALBUTTON 0x02 
#define MENUBUTTON   0x03 //A menu can have only 1 or 0 buttons selected.

struct buttonset {
   button *a;
   int numbuttons;
   int settype;
   bool firsttime;
   bool active;
};


void Initialize_Button_Set(int numbuttons, button *a, int def,
                           int settype, buttonset &set) {
   int c;
   bool onn;
   for (c=0; c < numbuttons; c++) {
      onn = (c == def)&&(settype != MENUBUTTON);
      a[c].buttonstate = onn;
   }
   set.a = a;
   set.numbuttons = numbuttons;
   set.settype = settype;
   set.firsttime = true;
   set.active = true;
   return;
}

void Check_Button_Set(int x, int y, bool b, buttonset &set,
                      bool &depressed, bool forcedraw=false) {
   int c, d;
   if (set.active) {

   switch (set.settype) {
      case RADIOBUTTON:

      for (c=0; c < set.numbuttons; c++) {
         if ((!set.a[c].buttonstate||set.firsttime)||forcedraw) {
            if (Process_Button(set.a[c],x,y,b,depressed,forcedraw)) {
               for (d=0; d < set.numbuttons; d++) {
                  if (d != c) {
                     set.a[d].buttonstate = false;
                     Process_Button(set.a[d],x,y,b,depressed);
                  }
               }
            }
         }
      }
      set.firsttime = false;
      break;

      case NORMALBUTTON:
      for (c=0; c < set.numbuttons; c++) {
         Process_Button(set.a[c],x,y,b,depressed,forcedraw);
      }
      break;

      case MENUBUTTON:
      for (c=0; c < set.numbuttons; c++) {
         if (Process_Button(set.a[c],x,y,b,depressed)) {
            for (d=0; d < set.numbuttons; d++) {
               if (d != c) {
                  set.a[d].buttonstate = false;
                  Process_Button(set.a[d],x,y,b,depressed);
               }
            }
         }
      }
      break;
      
   }

   }
   return;
}

void Mouse_Interface(int &x, int &y, bool &b, bool &depressed) {
   Check_Mouse(x, y, b);

   if (depressed && !b) depressed = false;
   return;
}

void Erase_Button(button a) {
   Hide_Cursor();
   Filled_Box(a.minx,a.miny,a.maxx,a.maxy,0);
   Show_Cursor();
   return;
}

void Erase_Button_Set(buttonset set) {
   int c;
   for (c=0; c < set.numbuttons; c++) {
      Erase_Button(set.a[c]);
   }
   return;
}

//Drop-down menus

struct dropdown {
   buttonset *a;
   bool active;
};

//Takes a button set, makes a drop-down menu from it
void Initialize_Drop_Down(buttonset *a, dropdown &b) {
   
   Erase_Button_Set(*a);

   b.a = a;
   b.active = false;
}

void Check_Drop_Down(int x, int y, bool b, bool &depressed, dropdown &dd,
   bool selected) {
   int c;

   if (selected) {
      if (dd.active) {
         Check_Button_Set(x,y,b,*(dd.a),depressed);
      } else {
         Check_Button_Set(x,y,b,*(dd.a),depressed,true);
         dd.active=true;
      }
   } else {
      if (dd.active) {
         Erase_Button_Set(*(dd.a));
         dd.active = false;
         for (c=0; c < (*(dd.a)).numbuttons; c++) {
            (*(dd.a)).a[c].buttonstate=false;  //Turn off all menu buttons when deselected
         }
      }
   }

   return;
}
