#include <stdio.h>
#include <stdlib.h>
#include <bios.h>
#include <dos.h>
#include <fcntl.h>

//PCX, from Teach Yourself Game Programming in 21 Days, modified to *WORK*

//PCX header
typedef struct pcx_header_typ
   {
   char manufacturer;   //Always 10
   char version;        //
   char encoding;       //Always 1, RLE
   char bits_per_pixel; //Eight in our case
   int x,y;             //Upper left corner of image
   int width,height;    //Size of image
   int horz_res;        //Pels in x direction
   int vert_res;        //Pels in y direction
   char ega_palette[48];//Ignore
   char reserved;       //Nothing
   char num_color_planes;//Number of planes of the image
   int bytes_per_line;  //
   int palette_type;    //Ignore
   char padding[58];    //Extra bytes
   } pcx_header, *pcx_header_ptr;

//The data structure for the PCX file

typedef struct pcx_picture_typ
   {
   pcx_header header;           //header
   RGB_color palette[256];      //VGA palette
   char far * buffer;
   } pcx_picture, *pcx_picture_ptr;

//PCX functions

void PCX_Load(char *filename, pcx_picture_ptr image, int enable_palette) {
   FILE *fp;
   int num_bytes,index;
   int x,y;
   long count;
   unsigned char data;
   char far *temp_buffer;

   //Open the file
   fp = fopen(filename,"rb");
   
   //Load the header
   temp_buffer = (char far *)image;

   for (index=0; index<128; index++) {
      temp_buffer[index] = (char)getc(fp);
   }

   //Load the data and decompress it into the buffer.
   count=0;
   x=0;
   y=0;

   while (count<=(64000)) {
      //Get the first piece of data
      data = (unsigned char)getc(fp);

      //Is this an RLE?
      if (data >=192) {
         //How many bytes in the run?
         num_bytes = data-192;

         //Get the actual data for the run.
         data = (unsigned char)getc(fp);

         //Replicate the data in the buffer num_bytes times.
         while (num_bytes-- >0) {
            SetPixel(x,y,data);
            x++;
            if (x==320) {x=0; y++;}
            count++;
         }

      } else {
         SetPixel(x,y,data);
         x++;
         if (x==320) {x=0; y++;}
         count++;
      }
   }

   //Move to the end of the file, then back up 768 bytes to the beginning of
   //the palette.
   fseek(fp,-768L,SEEK_END);

   //Load the palette into the VGA's palette registers.
   for (index=0; index<256; index++) {
      //Get the red component.
      image->palette[index].red=(unsigned char)(getc(fp) >> 2);
      //Get the green component.
      image->palette[index].green=(unsigned char)(getc(fp) >> 2);
      //Get the blue component.
      image->palette[index].blue=(unsigned char)(getc(fp) >> 2);
   }

   fclose(fp);

   //Change the palette to the newly loaded palette if commanded to do so.

   if (enable_palette) {
      //For each palette register, set to the new color values.

      for (index=0; index<256; index++) {
         Set_Palette_Register(index, image->palette[index]);
      }
   }

   return;
}

void Show_PCX(char * filename, bool fade) {
   pcx_picture pcx;
   
   Cls();

   Blank_Palette();

   PCX_Load(filename, (pcx_picture_ptr)&pcx, 0);

   if (fade) Fade_Into(pcx.palette);
   else Restore_Palette(pcx.palette);
   
   return;
}
